import { ReadableStreamController } from './ReadableStreamController';
import { ReadableStreamReader } from './ReadableStreamReader';
export interface ReadableStreamSource<Data> {
    start?(controller: ReadableStreamController<Data>): Promise<void>;
    pull?(): Promise<void>;
    cancel?(): any;
}
export interface ReadableStreamStrategy<Data> {
    highWaterMark?: number;
    size?: ReadableStreamSizeAlgorithm<Data>;
}
export declare type ReadableStreamSizeAlgorithm<Data> = (chunk: Data) => number;
export declare enum ReadableStreamState {
    Closed = "Closed",
    Errored = "Errored",
    Readable = "Readable"
}
/**
 * Readable stream.
 */
export declare class ReadableStream<Data> {
    locked: boolean;
    private state;
    private controller;
    private reader?;
    private storedError?;
    constructor(source?: ReadableStreamSource<Data>, strategy?: ReadableStreamStrategy<Data>);
    getReader(): Promise<ReadableStreamReader<Data>>;
    cancel(reason?: string): Promise<void>;
    pipeTo(): any;
    pipeThrough(): any;
    tee(): any;
    /** @see https://github.com/nodejs/node/blob/561f7fe941929d6c10b82b8250c04afb0693e4f3/lib/internal/webstreams/readablestream.js#L1559 */
    close(): void;
    private triggerError;
    private getSizeAlgorithm;
}
