"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReadableStream = exports.ReadableStreamState = void 0;
const ReadableStreamController_1 = require("./ReadableStreamController");
const ReadableStreamReader_1 = require("./ReadableStreamReader");
var ReadableStreamState;
(function (ReadableStreamState) {
    ReadableStreamState["Closed"] = "Closed";
    ReadableStreamState["Errored"] = "Errored";
    ReadableStreamState["Readable"] = "Readable";
})(ReadableStreamState = exports.ReadableStreamState || (exports.ReadableStreamState = {}));
/**
 * Readable stream.
 */
class ReadableStream {
    locked;
    state;
    controller;
    reader;
    storedError;
    constructor(source, strategy) {
        this.locked = false;
        this.state = ReadableStreamState.Readable;
        this.controller = new ReadableStreamController_1.ReadableStreamController({
            stream: this,
            startAlgorithm: source?.start || (async () => void 0),
            pullAlgorithm: source?.pull,
            cancelAlgorithm: source?.cancel,
            sizeAlgorithm: this.getSizeAlgorithm(strategy?.size),
            highWaterMark: strategy?.highWaterMark ?? 1,
        });
    }
    async getReader() {
        this.reader = new ReadableStreamReader_1.ReadableStreamReader(this);
        return this.reader;
    }
    async cancel(reason) {
        if (this.locked) {
            throw new Error('ReadableStream is locked');
        }
        /** @todo */
    }
    pipeTo() {
        return;
    }
    pipeThrough() {
        return;
    }
    tee() {
        return;
    }
    /** @see https://github.com/nodejs/node/blob/561f7fe941929d6c10b82b8250c04afb0693e4f3/lib/internal/webstreams/readablestream.js#L1559 */
    close() {
        this.state = ReadableStreamState.Closed;
        if (this.reader?.readRequests.length) {
            for (const readRequest of this.reader.readRequests) {
                readRequest.close();
            }
            this.reader.readRequests = [];
        }
    }
    triggerError(error) {
        if (this.state === ReadableStreamState.Readable) {
            this.state = ReadableStreamState.Errored;
            this.storedError = error;
            /**
             * @todo Reject reader promise, if present.
             * @see https://github.com/nodejs/node/blob/561f7fe941929d6c10b82b8250c04afb0693e4f3/lib/internal/webstreams/readablestream.js#L1591
             */
            if (this.reader?.readRequests.length) {
                for (const readRequest of this.reader.readRequests) {
                    readRequest.error(error);
                }
                this.reader.readRequests = [];
            }
        }
    }
    getSizeAlgorithm(size) {
        if (typeof size === 'undefined') {
            return () => 1;
        }
        return size;
    }
}
exports.ReadableStream = ReadableStream;
//# sourceMappingURL=ReadableStream.js.map