"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReadableStreamReader = void 0;
const ReadableStream_1 = require("./ReadableStream");
const ReadRequest_1 = require("./ReadRequest");
class ReadableStreamReader {
    stream;
    readRequests;
    constructor(stream) {
        this.stream = stream;
        this.readRequests = [];
        return this;
    }
    async read() {
        if (this.stream.locked) {
            throw new Error('ReadableStream is locked');
        }
        const readRequest = new ReadRequest_1.ReadRequest();
        switch (this.stream['state']) {
            case ReadableStream_1.ReadableStreamState.Closed: {
                readRequest.close();
                break;
            }
            case ReadableStream_1.ReadableStreamState.Errored: {
                readRequest.error(this.stream['storedError']);
                break;
            }
            case ReadableStream_1.ReadableStreamState.Readable: {
                this.stream['controller'].pull(readRequest);
                break;
            }
        }
        return readRequest.promise;
    }
    /** @see https://github.com/nodejs/node/blob/561f7fe941929d6c10b82b8250c04afb0693e4f3/lib/internal/webstreams/readablestream.js#L1533 */
    async cancel(reason) {
        switch (this.stream['state']) {
            case ReadableStream_1.ReadableStreamState.Closed: {
                return;
            }
            case ReadableStream_1.ReadableStreamState.Errored: {
                throw this.stream['storedError'];
            }
        }
        return this.stream['controller'].cancel(reason);
    }
    /** @see https://github.com/nodejs/node/blob/561f7fe941929d6c10b82b8250c04afb0693e4f3/lib/internal/webstreams/readablestream.js#L1718 */
    releaseLock() {
        if (this.readRequests.length) {
            throw new Error('Cannot release with pending read requests');
        }
        if (this.stream['state'] === ReadableStream_1.ReadableStreamState.Readable) {
            /** @todo */
        }
        this.stream['reader'] = undefined;
    }
}
exports.ReadableStreamReader = ReadableStreamReader;
//# sourceMappingURL=ReadableStreamReader.js.map